==> constrainedrandom-1.1.0.tar.gz: constrainedrandom-1.1.0/tests/features/errors.py <==
# SPDX-License-Identifier: MIT
# Copyright (c) 2023 Imagination Technologies Ltd. All Rights Reserved

'''
Test error cases.
'''

from random import Random

from constrainedrandom import RandObj, RandomizationError

from .basic import MultiSum
from .. import testutils


class ImpossibleThorough(MultiSum):
    '''
    Test the thorough solver for a problem that will always fail.

    The thorough solver almost always converges, so it's very
    hard to construct a problem 'too hard' for it.
    Instead, make it impossible and only enable the thorough
    solver.
    '''

    EXPECTED_ERROR_RAND = RandomizationError

    def get_randobj(self, *args):
        randobj = super().get_randobj(*args)
        # Only use thorough solver.
        randobj.set_solver_mode(naive=False, sparse=False)
        # Make problem impossible so that numbers can't sum to 41.
        def mod_2(z):
            return z % 2 == 0
        randobj.add_constraint(mod_2, ('x',))
        randobj.add_constraint(mod_2, ('y',))
        randobj.add_constraint(mod_2, ('z',))
        # Setting max domain size to 1 makes this run a bit quicker.
        randobj._max_domain_size = 1
        return randobj


class ImpossibleOneVar(testutils.RandObjTestBase):
    '''
    Test an impossible constraint problem with one variable.
    '''

    def get_randobj(self, *args):
        randobj = RandObj(*args)
        def eq_zero(x):
            return x == 0
        randobj.add_rand_var('a', domain=[1,], constraints=[eq_zero,])
        return randobj

    def test_randobj(self):
        # Override the whole test function, because unsolvable
        # simple single variables fail on creation, rather
        # than when randomize is called.
        self.assertRaises(RandomizationError, self.get_randobj, 0)
        self.assertRaises(RandomizationError, self.get_randobj, 1)


class ImpossibleComplexVar(testutils.RandObjTestBase):
    '''
    Test an impossible constraint problem with one variable, where
    the variable state space is too large to fail on creation.
    '''

    EXPECTED_ERROR_RAND = RandomizationError

    def get_randobj(self, *args):
        randobj = RandObj(*args)
        def eq_minus_one(x):
            return x == -1
        randobj.add_rand_var('a', bits=64, constraints=[eq_minus_one,])
        return randobj


class ImpossibleMultiVar(testutils.RandObjTestBase):
    '''
    Test an impossible constraint problem with multiple variables.
    '''

    EXPECTED_ERROR_RAND = RandomizationError

    def get_randobj(self, *args):
        randobj = RandObj(*args)
        def lt_5(x):
            return x < 5
        randobj.add_rand_var('a', domain=range(10), constraints=[lt_5,])
        randobj.add_rand_var('b', domain=range(10), constraints=[lt_5,])
        def sum_gt_10(x, y):
            return x + y > 10
        randobj.add_constraint(sum_gt_10, ('a', 'b'))
        return randobj


class NegativeLength(testutils.RandObjTestBase):
    '''
    Test a random list with negative length.
    '''

    EXPECTED_ERROR_INIT = AssertionError

    def get_randobj(self, *args):
        randobj = RandObj(*args)
        randobj.add_rand_var('bad_list', bits=1, length=-1)
        return randobj


class NegativeRandLength(testutils.RandObjTestBase):
    '''
    Test a random list with negative random length.
    '''

    EXPECTED_ERROR_RAND = AssertionError

    def get_randobj(self, *args):
        randobj = RandObj(*args)
        randobj.add_rand_var('bad_length', domain=range(-10,-1))
        randobj.add_rand_var('bad_list', bits=1, rand_length='bad_length')
        return randobj


class EmptyListDependent(testutils.RandObjTestBase):
    '''
    Test a random length list which is empty
    but depended on by another variable.
    '''

    EXPECTED_ERROR_RAND = RandomizationError

    def get_randobj(self, *args):
        randobj = RandObj(*args)
        randobj.add_rand_var('length', domain=[0])
        randobj.add_rand_var('list', bits=4, rand_length='length')
        randobj.add_rand_var('list_member', bits=4)
        def in_list_c(x, y):
            return x in y
        randobj.add_constraint(in_list_c, ('list_member', 'list'))
        return randobj

==> constrainedrandom-1.1.2.tar.gz: constrainedrandom-1.1.2/tests/features/errors.py <==
# SPDX-License-Identifier: MIT
# Copyright (c) 2023 Imagination Technologies Ltd. All Rights Reserved

'''
Test error cases.
'''

from random import Random

from constrainedrandom import RandObj, RandomizationError

from .basic import MultiSum
from .. import testutils


class ImpossibleThorough(MultiSum):
    '''
    Test the thorough solver for a problem that will always fail.

    The thorough solver almost always converges, so it's very
    hard to construct a problem 'too hard' for it.
    Instead, make it impossible and only enable the thorough
    solver.
    '''

    EXPECTED_ERROR_RAND = RandomizationError

    def get_randobj(self, *args):
        randobj = super().get_randobj(*args)
        # Only use thorough solver.
        randobj.set_solver_mode(naive=False, sparse=False)
        # Make problem impossible so that numbers can't sum to 41.
        def mod_2(z):
            return z % 2 == 0
        randobj.add_constraint(mod_2, ('x',))
        randobj.add_constraint(mod_2, ('y',))
        randobj.add_constraint(mod_2, ('z',))
        # Setting max domain size to 1 makes this run a bit quicker.
        randobj._max_domain_size = 1
        return randobj


class ImpossibleOneVar(testutils.RandObjTestBase):
    '''
    Test an impossible constraint problem with one variable.
    '''

    def get_randobj(self, *args):
        randobj = RandObj(*args)
        def eq_zero(x):
            return x == 0
        randobj.add_rand_var('a', domain=[1,], constraints=[eq_zero,])
        return randobj

    def test_randobj(self):
        # Override the whole test function, because unsolvable
        # simple single variables fail on creation, rather
        # than when randomize is called.
        self.assertRaises(RandomizationError, self.get_randobj, 0)
        self.assertRaises(RandomizationError, self.get_randobj, 1)


class ImpossibleComplexVar(testutils.RandObjTestBase):
    '''
    Test an impossible constraint problem with one variable, where
    the variable state space is too large to fail on creation.
    '''

    EXPECTED_ERROR_RAND = RandomizationError

    def get_randobj(self, *args):
        randobj = RandObj(*args)
        def eq_minus_one(x):
            return x == -1
        randobj.add_rand_var('a', bits=64, constraints=[eq_minus_one,])
        return randobj


class ImpossibleMultiVar(testutils.RandObjTestBase):
    '''
    Test an impossible constraint problem with multiple variables.
    '''

    EXPECTED_ERROR_RAND = RandomizationError

    def get_randobj(self, *args):
        randobj = RandObj(*args)
        def lt_5(x):
            return x < 5
        randobj.add_rand_var('a', domain=range(10), constraints=[lt_5,])
        randobj.add_rand_var('b', domain=range(10), constraints=[lt_5,])
        def sum_gt_10(x, y):
            return x + y > 10
        randobj.add_constraint(sum_gt_10, ('a', 'b'))
        return randobj


class NegativeLength(testutils.RandObjTestBase):
    '''
    Test a random list with negative length.
    '''

    EXPECTED_ERROR_INIT = ValueError

    def get_randobj(self, *args):
        randobj = RandObj(*args)
        randobj.add_rand_var('bad_list', bits=1, length=-1)
        return randobj


class NegativeRandLength(testutils.RandObjTestBase):
    '''
    Test a random list with negative random length.
    '''

    EXPECTED_ERROR_RAND = ValueError

    def get_randobj(self, *args):
        randobj = RandObj(*args)
        randobj.add_rand_var('bad_length', domain=range(-10,-1))
        randobj.add_rand_var('bad_list', bits=1, rand_length='bad_length')
        return randobj


class EmptyListDependent(testutils.RandObjTestBase):
    '''
    Test a random length list which is empty
    but depended on by another variable.
    '''

    EXPECTED_ERROR_RAND = RandomizationError

    def get_randobj(self, *args):
        randobj = RandObj(*args)
        randobj.add_rand_var('length', domain=[0])
        randobj.add_rand_var('list', bits=4, rand_length='length')
        randobj.add_rand_var('list_member', bits=4)
        def in_list_c(x, y):
            return x in y
        randobj.add_constraint(in_list_c, ('list_member', 'list'))
        return randobj
